# -*- coding: utf-8 -*-
"""
Created on Mon Aug 20 17:51:30 2018
@author: mshahnazari

Updated by The Lantau Group (Tom Wilson, Paul Buckland)
"""

# Import Python libraries
import pandas as pd
from pandas import ExcelWriter
import time
import numpy as np

# Time the computations
start_time = time.time()

# Read the outage rate excel file as a dataframe
dataOR = pd.read_excel("Input_EFOR.xlsx", header=0, index_col=0, engine='openpyxl')
dataOR.dropna(axis=0, how='all', inplace=True)

# Label the columns of COPT
COPT_Labels = {'outageState': None,
               'availableCapacity': None,
               'outageCapacity': None,
               'Pn-1': None,
               'Pn-1(X-C)': None,
               'X-C': None,
               'exactProbability': None,
               'cumulativeProbability': None
               }

# Total installed capacity of resources included in the COPT
systemCapacity = dataOR['Capacity Credits (MW)'].sum()

# Total number of rows in the COPT
COPT_rowCount = int(systemCapacity) + 1

# Initialise the COPT with zeros and column labels
COPT_output = pd.DataFrame(np.zeros((COPT_rowCount, len(COPT_Labels.keys()))),
                           columns=COPT_Labels.keys())

# Variable to define COPT interaction steps
outage_step = 1

# Step increment for indicating the number of interactions
outage_state = 1

# Auxiliary variable
tempCap = 0

# Row count
rowNum = 0
print('')

for index, irow in dataOR.iterrows():

    # Progress bar
    rowNum += 1
    print('Unit ' + str(rowNum) + ' of ' + str(len(dataOR.index)) + ' being processed...')

    newCapacity = irow['Capacity Credits (MW)']
    FOR_newCapacity = irow['Forced Outage Rate']
    tempCap += newCapacity
    outageCapacity = 0

    # Changed while loop condition to ensure COPT is calculated for all outage capacity scenarios
    while outageCapacity <= systemCapacity:

        COPT_output.loc[outageCapacity]['outageState'] = outage_state
        COPT_output.loc[outageCapacity]['availableCapacity'] = tempCap - outageCapacity
        COPT_output.loc[outageCapacity]['outageCapacity'] = outageCapacity
        COPT_output.loc[outageCapacity]['X-C'] = outageCapacity - newCapacity

        if outageCapacity == 0:
            COPT_output.loc[outageCapacity]['Pn-1'] = 1  # The cumulative probability Pn-1(outage>=0) = 1
        else:
            COPT_output.loc[outageCapacity]['Pn-1'] = COPT_output.loc[outageCapacity]['cumulativeProbability']

        if COPT_output.loc[outageCapacity]['X-C'] <= 0:
            COPT_output.loc[outageCapacity]['Pn-1(X-C)'] = 1
        else:
            aux_idx = COPT_output['outageCapacity'] == COPT_output.loc[outageCapacity]['X-C']
            COPT_output.loc[outageCapacity]['Pn-1(X-C)'] = COPT_output.loc[aux_idx]['Pn-1']

        COPT_output.loc[outageCapacity]['cumulativeProbability'] = COPT_output.loc[outageCapacity]['Pn-1'] * (
                    1 - FOR_newCapacity) + COPT_output.loc[outageCapacity]['Pn-1(X-C)'] * FOR_newCapacity

        if outageCapacity > 0:
            COPT_output.loc[outageCapacity - 1]['exactProbability'] = COPT_output.loc[outageCapacity - 1][
                                                                          'cumulativeProbability'] - \
                                                                      COPT_output.loc[outageCapacity][
                                                                          'cumulativeProbability']

        outageCapacity += outage_step
        outage_state += 1

# Write outputs
writer = ExcelWriter("Working_COPT.xlsx")
COPT_output.to_excel(writer, "COPT", index=False)
writer.save()

# Report time taken
print("\nTime taken: %d seconds" % (time.time() - start_time))